#include "pcg_random.hpp"

int main()
{
    std::cout << "checking seed 0 to 9 using pcg32_fast.." << std::endl;
    for (int seed = 0; seed < 10; ++seed) {
        auto rng = pcg32_fast(seed);
        std::cout << "\t" << seed << "\t" << rng() << "\t" << rng() << "\t" << rng() << std::endl;
    }

    std::cout << "checking seed 0 to 9 using pcg64_fast.." << std::endl;
    for (int seed = 0; seed < 10; ++seed) {
        auto rng = pcg64_fast(seed);
        std::cout << "\t" << seed << "\t" << rng() << "\t" << rng() << "\t" << rng() << std::endl;
    }

    std::cout << "checking seed 0 to 9 using pcg32.." << std::endl;
    for (int seed = 0; seed < 10; ++seed) {
        auto rng = pcg32(seed);
        std::cout << "\t" << seed << "\t" << rng() << "\t" << rng() << "\t" << rng() << std::endl;
    }

    std::cout << "checking seed 0 to 9 using pcg64.." << std::endl;
    for (int seed = 0; seed < 10; ++seed) {
        auto rng = pcg64(seed);
        std::cout << "\t" << seed << "\t" << rng() << "\t" << rng() << "\t" << rng() << std::endl;
    }
    std::cout << "It appears that that seed = 4*n+i for i in {0,1,2,3} gives identical results for a fixed n for the two fast variants. Additionally, the 32 bit fast variant seems to start at zero." << std::endl;

    std::cout << "Checking whether the problem remains for higher seeds:" << std::endl;
    int problemRemainsConsistently = 1;
    for (unsigned int seed = 0; seed < 1e9; ++seed) {
        auto a = pcg64_fast(4 * seed);
        auto b = pcg64_fast(4 * seed + 1);
        auto c = pcg64_fast(4 * seed + 2);
        auto d = pcg64_fast(4 * seed + 3);
        if (a != b || a != c || a != d || b != c || b != d || c != d)
        {
            std::cout << 4 * seed << std::endl;
            problemRemainsConsistently = 0;
        }
    }
    std::cout << "pcg64_fast: All checked generators are memory identical for fixed seed // 4" << std::endl;
    problemRemainsConsistently = 1;
    for (unsigned int seed = 0; seed < 1e9; ++seed) {
        auto a = pcg32_fast(4 * seed);
        auto b = pcg32_fast(4 * seed + 1);
        auto c = pcg32_fast(4 * seed + 2);
        auto d = pcg32_fast(4 * seed + 3);
        if (a != b || a != c || a != d || b != c || b != d || c != d)
        {
            std::cout << 4 * seed << std::endl;
            problemRemainsConsistently = 0;
        }
    }
    std::cout << "pcg32_fast: All checked generators are memory identical for fixed seed // 4" << std::endl;
}